// Simple SceneController 

#include <MySensor.h>
#include <SPI.h>
#include <Bounce2.h>

//Relay Info
#define RELAY_1  14  // Arduino Digital I/O pin number for first relay (second on pin+1 etc)
#define NUMBER_OF_RELAYS 5 // Total number of attached relays
#define RELAY_ON 1  // GPIO value to write to turn on attached relay
#define RELAY_OFF 0 // GPIO value to write to turn off attached relay

//Scene Controller Info
#define CHILD_ID 3
// PIN for the buttons
byte buttons[] = {3, 4, 5, 6, 7, 8};
#define NUMBUTTONS sizeof(buttons)
byte pressed[NUMBUTTONS], justpressed[NUMBUTTONS], justreleased[NUMBUTTONS];

MySensor gw;
Bounce debouncer[NUMBUTTONS];

int oldValue[NUMBUTTONS];

MyMessage msgOn(CHILD_ID,V_SCENE_ON);
MyMessage msgOff(CHILD_ID,V_SCENE_OFF);

void setup()  
{  
  // Initialize library and add callback for incoming messages
  gw.begin(incomingMessage, AUTO, true);

  // Relay SETUP
  // Fetch relay status
  for (int sensor=1, pin=RELAY_1; sensor<=NUMBER_OF_RELAYS;sensor++, pin++) {
    // Register all sensors to gw (they will be created as child devices)
    gw.present(sensor, S_LIGHT);
    // Then set relay pins in output mode
    pinMode(pin, OUTPUT);   
    // Set relay to last known state (using eeprom storage) 
    digitalWrite(pin, gw.loadState(sensor)?RELAY_ON:RELAY_OFF);
  }
  
  //Scene Controller SETUP
  /// Make input & enable pull-up resistors on switch pins
  for (short i=0; i < NUMBUTTONS; i++){

    pinMode(buttons[i], INPUT);
    digitalWrite(buttons[i], HIGH);
    oldValue[i] = -1;
    // After setting up the button, setup debouncer
    debouncer[i].attach(buttons[i]);
    debouncer[i].interval(5);

  }



  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo("Scene and Relay", "1.0");
  // Register binary input sensor to gw (they will be created as child devices)
  gw.present(CHILD_ID, S_SCENE_CONTROLLER); 
}


//  Check if digital input has changed and send in new value
void loop() 
{
  for (short i=0; i < NUMBUTTONS; i++){
    
    // Alway process incoming messages whenever possible
    gw.process();
    
    debouncer[i].update();
    // Get the update value
    int value = debouncer[i].read();

    if (value != oldValue[i]) {
       // Send in the new value
       if (value==HIGH) {
         gw.send(msgOff.set(i));
       }
       else {
         gw.send(msgOn.set(i));
       }
       oldValue[i] = value;
    }
  }
}

void incomingMessage(const MyMessage &message) {
  // We only expect one type of message from controller. But we better check anyway.
  if (message.type==V_LIGHT) {
     // Change relay state
     digitalWrite(message.sensor-1+RELAY_1, message.getBool()?RELAY_ON:RELAY_OFF);
     // Store state in eeprom
     gw.saveState(message.sensor, message.getBool());
     // Write some debug info
     Serial.print("Incoming change for sensor:");
     Serial.print(message.sensor);
     Serial.print(", New status: ");
     Serial.println(message.getBool());
   } 
}
